package com.ruoyi.system.service.impl;

import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.domain.SysRoleDept;
import com.ruoyi.system.domain.SysRoleMenu;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.SpringUtils;
import java.util.Collection;
import java.util.Arrays;
import com.ruoyi.common.core.utils.StringUtils;
import java.util.HashSet;
import java.util.Set;
import java.util.Iterator;
import com.ruoyi.common.datascope.annotation.DataScope;
import java.util.List;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.mapper.SysRoleDeptMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.mapper.SysRoleMenuMapper;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SysRoleMapper;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISysRoleService;

@Service
public class SysRoleServiceImpl implements ISysRoleService
{
    @Autowired
    private SysRoleMapper roleMapper;
    @Autowired
    private SysRoleMenuMapper roleMenuMapper;
    @Autowired
    private SysUserRoleMapper userRoleMapper;
    @Autowired
    private SysRoleDeptMapper roleDeptMapper;
    
    @DataScope(deptAlias = "d")
    public List<SysRole> selectRoleList(final SysRole role) {
        return this.roleMapper.selectRoleList(role);
    }
    
    public List<SysRole> selectRolesByUserId(final Long userId) {
        final List<SysRole> userRoles = this.roleMapper.selectRolePermissionByUserId(userId);
        final List<SysRole> roles = this.selectRoleAll();
        for (final SysRole role : roles) {
            for (final SysRole userRole : userRoles) {
                if (role.getRoleId() == (long)userRole.getRoleId()) {
                    role.setFlag(true);
                    break;
                }
            }
        }
        return roles;
    }
    
    public Set<String> selectRolePermissionByUserId(final Long userId) {
        final List<SysRole> perms = this.roleMapper.selectRolePermissionByUserId(userId);
        final Set<String> permsSet = new HashSet<String>();
        for (final SysRole perm : perms) {
            if (StringUtils.isNotNull((Object)perm)) {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }
    
    public List<SysRole> selectRoleAll() {
        return ((SysRoleServiceImpl)SpringUtils.getAopProxy((Object)this)).selectRoleList(new SysRole());
    }
    
    public List<Long> selectRoleListByUserId(final Long userId) {
        return this.roleMapper.selectRoleListByUserId(userId);
    }
    
    public SysRole selectRoleById(final Long roleId) {
        return this.roleMapper.selectRoleById(roleId);
    }
    
    public boolean checkRoleNameUnique(final SysRole role) {
        final Long roleId = StringUtils.isNull((Object)role.getRoleId()) ? -1L : role.getRoleId();
        final SysRole info = this.roleMapper.checkRoleNameUnique(role.getRoleName());
        return !StringUtils.isNotNull((Object)info) || info.getRoleId() == (long)roleId;
    }
    
    public boolean checkRoleKeyUnique(final SysRole role) {
        final Long roleId = StringUtils.isNull((Object)role.getRoleId()) ? -1L : role.getRoleId();
        final SysRole info = this.roleMapper.checkRoleKeyUnique(role.getRoleKey());
        return !StringUtils.isNotNull((Object)info) || info.getRoleId() == (long)roleId;
    }
    
    public void checkRoleAllowed(final SysRole role) {
        if (StringUtils.isNotNull((Object)role.getRoleId()) && role.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
    }
    
    public void checkRoleDataScope(final Long... roleIds) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            for (final Long roleId : roleIds) {
                final SysRole role = new SysRole();
                role.setRoleId(roleId);
                final List<SysRole> roles = ((SysRoleServiceImpl)SpringUtils.getAopProxy((Object)this)).selectRoleList(role);
                if (StringUtils.isEmpty((Collection)roles)) {
                    throw new ServiceException("没有权限访问角色数据！");
                }
            }
        }
    }
    
    public int countUserRoleByRoleId(final Long roleId) {
        return this.userRoleMapper.countUserRoleByRoleId(roleId);
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int insertRole(final SysRole role) {
        this.roleMapper.insertRole(role);
        return this.insertRoleMenu(role);
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int updateRole(final SysRole role) {
        this.roleMapper.updateRole(role);
        this.roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
        return this.insertRoleMenu(role);
    }
    
    public int updateRoleStatus(final SysRole role) {
        return this.roleMapper.updateRole(role);
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int authDataScope(final SysRole role) {
        this.roleMapper.updateRole(role);
        this.roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        return this.insertRoleDept(role);
    }
    
    public int insertRoleMenu(final SysRole role) {
        int rows = 1;
        final List<SysRoleMenu> list = new ArrayList<SysRoleMenu>();
        for (final Long menuId : role.getMenuIds()) {
            final SysRoleMenu rm = new SysRoleMenu();
            rm.setRoleId(role.getRoleId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        if (list.size() > 0) {
            rows = this.roleMenuMapper.batchRoleMenu((List)list);
        }
        return rows;
    }
    
    public int insertRoleDept(final SysRole role) {
        int rows = 1;
        final List<SysRoleDept> list = new ArrayList<SysRoleDept>();
        for (final Long deptId : role.getDeptIds()) {
            final SysRoleDept rd = new SysRoleDept();
            rd.setRoleId(role.getRoleId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0) {
            rows = this.roleDeptMapper.batchRoleDept((List)list);
        }
        return rows;
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int deleteRoleById(final Long roleId) {
        this.roleMenuMapper.deleteRoleMenuByRoleId(roleId);
        this.roleDeptMapper.deleteRoleDeptByRoleId(roleId);
        return this.roleMapper.deleteRoleById(roleId);
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int deleteRoleByIds(final Long[] roleIds) {
        for (final Long roleId : roleIds) {
            this.checkRoleAllowed(new SysRole(roleId));
            this.checkRoleDataScope(roleId);
            final SysRole role = this.selectRoleById(roleId);
            if (this.countUserRoleByRoleId(roleId) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        this.roleMenuMapper.deleteRoleMenu(roleIds);
        this.roleDeptMapper.deleteRoleDept(roleIds);
        return this.roleMapper.deleteRoleByIds(roleIds);
    }
    
    public int deleteAuthUser(final SysUserRole userRole) {
        return this.userRoleMapper.deleteUserRoleInfo(userRole);
    }
    
    public int deleteAuthUsers(final Long roleId, final Long[] userIds) {
        return this.userRoleMapper.deleteUserRoleInfos(roleId, userIds);
    }
    
    public int insertAuthUsers(final Long roleId, final Long[] userIds) {
        final List<SysUserRole> list = new ArrayList<SysUserRole>();
        for (final Long userId : userIds) {
            final SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return this.userRoleMapper.batchUserRole((List)list);
    }
}
